/**===========================================================================
 * Name        : welderstation.ino
 * Author      : Maddox's LAB
 * Version     : 1.0.2.1
 * License     : Creative Commons Attribution 4.0 International License
 * Description : Logic for an electronic soldering station
 *============================================================================
 *
 * This work is licensed under the Creative Commons Attribution 4.0 International License. 
 * To view a copy of this license, visit http://creativecommons.org/licenses/by/4.0/ or 
 * send a letter to Creative Commons, PO Box 1866, Mountain View, CA 94042, USA.
 **/
 

#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306_i2c.h>
#include "max6675.h"
#include "Memory.h"


///PIN DEFINITIONS
#define RT_SCK 2
#define RT_DT 4
#define RT_SW 3
#define TC_SCK 13
#define TC_DATA 12
#define TC_EN 10
#define HEAT 9
#define OLED_RESET 4
#define FAN 5

#define HEATING_TEMP_DELTA 24


///Global Variables
uint64_t lastRotaryTick;
uint64_t lastSwitchTick;
bool isPower; //If heater circuit is On or Off
bool isHeating; //If actual heatting is On or Off
float t_read; //Temperature
uint16_t t_set; //Set-Temperature
uint64_t measureTime; //Next time to measure
bool toRefresh;
uint64_t refreshingTime; //Next time to refresh screen
bool calibrationMode; //If is in calibration mode or not
int16_t d_cal; //Delta of temperature from sensor to shell

///Instances
Adafruit_SSD1306 display(OLED_RESET);
max6675 thermocouple(TC_EN, TC_DATA, TC_SCK);
Memory mem;

void setup(){
  //Init controls
  pinMode(RT_SCK, INPUT);
  pinMode(RT_DT, INPUT);
  pinMode(RT_SW, INPUT_PULLUP);
  attachInterrupt(digitalPinToInterrupt(RT_SCK), ISR_rotayTick, FALLING);
  attachInterrupt(digitalPinToInterrupt(RT_SW), ISR_switch, FALLING);
  lastRotaryTick = 0;
  lastSwitchTick = 0;
  
  pinMode(HEAT, OUTPUT);
  digitalWrite(HEAT, LOW);
  
  pinMode(FAN, OUTPUT);
  //analogWrite(FAN, 180); //Set the PSU fan at 50%
  digitalWrite(FAN, HIGH);
  
  isPower = false;
  isHeating = false;
  t_read = thermocouple.readTemp();
  t_set = 300;
  measureTime = millis()+100;
  
  calibrationMode = false;
  d_cal = mem.getCal(); //Load calibration from EEPROM

  toRefresh = true;
  refreshingTime = millis() + 500;

  // by default, we'll generate the high voltage from the 3.3v line internally! (neat!)
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x64)
  // init done
}


void loop(){
  if (millis() >= measureTime){
    t_read = thermocouple.readTemp();
    
    if (isHeating)
      t_read += HEATING_TEMP_DELTA; //Compensate the temp drop when is heating
      
    t_read -= d_cal; //Compensate temp with the calibrated delta
    measureTime = millis()+200;

    if (isPower){ //If is powered-on
      //Set heatting from T_read
      if (t_read < t_set)
        Heat(true);
      else //if (t_read >= t_set)
        Heat(false);
    }
    else
      Heat(false);
  }


  if (millis() >= refreshingTime)
      toRefresh = true;


      
  if (toRefresh){
    if (!calibrationMode)
      refreshDisplay(t_set, t_read, isPower);
    else
      calibrateDisplay(t_set, t_read, d_cal);
  }
  
  
  
}








//Function to activate or deactivate the welder heating
void Heat(bool isOn){
  if (isOn)
    digitalWrite(HEAT, HIGH);
  else
    digitalWrite(HEAT, LOW);

  isHeating = isOn;
}






void ISR_rotayTick(){
  bool clockwise = !digitalRead(RT_DT);

  if(millis() <= (lastRotaryTick+40)){ //Avoid switch capacitance oscillations
    ;
  }
  else if(millis() <= (lastRotaryTick+80)){ //Control velocity (fast)
    if (!calibrationMode)
      t_set = clockwise ? t_set+5 : t_set-5;
    else
      d_cal = clockwise ? d_cal+5 : d_cal-5;
  }
  else{ //velocity (slow)
    if (!calibrationMode)
      t_set = clockwise ? t_set+1 : t_set-1;
    else
      d_cal = clockwise ? d_cal+1 : d_cal-1;
  }

  toRefresh = true;
  lastRotaryTick = millis();
}


void ISR_switch(){
  if (millis() >= (lastSwitchTick+200)){ //Avoid switch capacitance oscillations
    if (calibrationMode){ 
      calibrationMode = false;
      mem.setCal(d_cal); //Save calibration settings in EEPROM
    }
    else { //If not in calibration
      isPower = !isPower;

      
      
      if (millis() <= (lastSwitchTick+500)){ //Check for fast double pulse (calibration mode)
        calibrationMode = true;
      }
      
    }
  }

  toRefresh = true;
  lastSwitchTick = millis();
}



void refreshDisplay(uint16_t set, float temp, bool power){
  display.setTextSize(2);
  display.setTextColor(WHITE);
  
  display.clearDisplay();
  display.setCursor(0, 0);
  display.print("SET:");
  display.print(set);
  display.setCursor(0, 20);
  
  if (temp > 6000.){
    display.print("No-Connect");
  }else{
    if (temp < 0.)
      temp = 0;
    display.print("TMP:");
    display.print(temp, 1);
  }
  
  
  if (!power){
    display.setCursor(20, 45);
    display.print("<<OFF>>");
  }

  
  display.display();

  refreshingTime = millis() + 500;
  toRefresh = false;
}




void calibrateDisplay(uint16_t set, float temp, int16_t delta){
  display.setTextSize(2);
  display.setTextColor(WHITE);
  
  display.clearDisplay();
  display.setCursor(0, 0);
  display.print("SET:");
  display.print(set);
  display.setCursor(0, 20);
  if (temp == 6666.)
    display.print("No-Connect");
  else{
    //if (temp < 0.)
    //  temp = 0;
    display.print("TMP:");
    display.print(temp, 1);
  }


  display.setCursor(0, 40);
  display.print("#CAL:");
  display.print(delta);
  
  
  display.display();

  refreshingTime = millis() + 500;
  toRefresh = false;
}
